// MONITOR TEPLOTY CHLADNICKY A MRAZNICKY v2.0 MODBUS
// 2x DS18B20, HD44780 LCD
// HTTP://WWW.PABLOX.NET
// original: 07/2017
// update: 9/2022 
// pridana hystereza vypnutia alarmu
// odstranene slucky tvorene delay, nahradene millis()
// doplnene rozhranie RS485 s MODBUS


#include <SoftwareSerial.h>
#include <SimpleModbusSlaveSoftwareSerial.h>
#include <OneWire.h>
#include <DallasTemperature.h>
#include <LiquidCrystal.h>

#define deviceID      7       // ADRESA PODSTANICE, ZMENIT PODLA POTREBY

// pripojenie modulu s MAX485
#define RX            3       // pripojit na pin RO MAX485
#define TX            5       // pripojit na pin DI MAX485
#define RS485_EN      4       // pripojit na pin RE/DE MAX485
#define BAUD_RATE     9600    // rychlost komunikacie

// REGISTRE PRE MODBUS RTU
enum 
{     
  ID, // 400001 ID podstanice
  TEMP1, // 400002 teplota chladnicka
  TEMP1SIGN, // 400003 znamienko teploty chladnicka: 0-zaporna, 1-kladna
  TEMP2, // 400004 teplota mraznicka 
  TEMP2SIGN,  // 400005 znamienko teploty mraznicka: 0-zaporna, 1-kladna   
  OK, // 400006 stav monitora, 1-OK, 0-chyba
  ALARM, // 400007 stav alarmu, 1-ALARM, 0-ok
  SENSOR, // 400008 stav snimacov, 0-OK, 1-chyba 
  TOTAL_ERRORS, // 400009 pocet chyb komunikacie od spustenia podstanice
  // ponechat
  TOTAL_REGS_SIZE 
};

unsigned int holdingRegs[TOTAL_REGS_SIZE]; 

SoftwareSerial SWserial(RX, TX);

// Pripojenie LCD [RS E D4 D5 D6 D7] 
LiquidCrystal lcd(12, 11, 10, 9, 8, 7);

// Cidla DS18B20 na D6
#define ONE_WIRE_BUS 6

OneWire oneWire(ONE_WIRE_BUS);
DallasTemperature sensors(&oneWire);

//=============================== UZIVATELOM DEFINOVANE PREMENNE PRE NASTAVENIE ===================  
//adresy jednotlivych cidiel, zistit pomocou prilozeneho programu
DeviceAddress chladnicka_cidlo = { 0x10, 0x5C, 0x8D, 0x02, 0x01, 0x08, 0x00, 0x55 };
DeviceAddress mraznicka_cidlo = { 0x10, 0x8F, 0x0C, 0x24, 0x01, 0x08, 0x00, 0xB7 };

const long  AlarmOdlozInterval = 5400000; // cas v ms pre odlozenie alarmu po stlaceni tlacidla, v tomto pripade 90min
int hystereza = 5; // hystereza pre skoncenie alarmu (teplota musi klesnut pod hodnotu definovanu ako alarmovaTeplota-hystereza)

// PREVADZKOVE HODNOTY TEPLOT (sluzi len na vypocet odchylky alarmovej/prevadzkovej teploty a zobrazenie na LCD)
int chladnickaNormTepl = 11;
int mraznickaNormTepl = -15;

// HODNOTY TEPLOT, KEDY SA SPUSTI ALARM
int chladnickaAlarmTepl = 18;
int mraznickaAlarmTepl = -10;

//================================================================================================

// premenne pre upravu teplot pre registre 
int temp1, temp2 = 0;

// casove premenne pre odlozenie alarmu 
unsigned long casOdlozeniaAlarmu = 0;
unsigned long currentMillis = 0;
int alarmNavratCas = 0;

// casove premenne pre obsluhu piezo bzuciaka
const int pipakInterval = 1000; 
unsigned long previousMillisPipak = 0;
unsigned long currentMillisPipak = 0;
bool pipakStav = 0;

// casove premenne pre blikanie LED
unsigned long previousMillisLED = 0;
unsigned long currentMillisLED = 0;
bool ledStav = 0;

bool chybaCidla = 0;
bool tlacidloStav;

//alarmove premenne
bool alarm = 0;
bool alarmPotvrdeny = 0;

//============================================= DEFINICIA IO PINOV ==========================================
const int tlacidloPin = A4; // potvrdzovacie tlacidlo alarmu, proti GND
const int ledPinCervena = A3;     // alarmova LED, proti GND
const int pipakPin =  2; // samovybuzujuce piezo, proti GND
const int ledPinZelena = A5; // alarmovy vystup, napriklad na GSM branu. V normalnom stave je v LOG.1, pri alarme prejde do LOG.0 

//premenne pre teploty
int teplotaChladnicka = 0;
int teplotaMraznicka = 0;

//=============================== SPECIALNE ZNAKY NA LCD ====================================
byte znakTeplomer[8] = { 0b00100, 0b01010,	0b01010,	0b01110,	0b11111,	0b11111,	0b01110,	0b00000 };
byte znakHodiny[8] = {	0b00000,	0b01110,	0b10101,	0b10111,	0b10001,	0b01110,	0b00000,	0b00000};
byte znakCelsius[8] = {	0b00000,	0b11000,	0b11011,	0b00100,	0b00100,	0b00100,	0b00011,	0b00000};
byte znakDelta[8] = { 0b00000,	0b00100,	0b00100,	0b01110,	0b01110,	0b11111,	0b11111,	0b00000};


void setup(void)
{ 
    // inicializacia MODBUS
  modbus_configure(&SWserial, BAUD_RATE, deviceID, RS485_EN, TOTAL_REGS_SIZE);
   
  Serial.begin(9600);
  
  //========nastavenie funkcie pinov pre tlacidlo, LEDku a bzuciak; vsetko pripojene proti GND
  pinMode(ledPinCervena, OUTPUT);
  pinMode(pipakPin, OUTPUT);
  pinMode(ledPinZelena, OUTPUT);
  pinMode(tlacidloPin, INPUT_PULLUP);

// inicializacia zbernice ONEWIRE
  sensors.begin();
// nastavenie rozlisenia cidiel na 10 bitov
  sensors.setResolution(chladnicka_cidlo, 10);
  sensors.setResolution(mraznicka_cidlo, 10);

// definicia vlastnych znakov na LCD
  lcd.createChar(0, znakTeplomer);
  lcd.createChar(2, znakHodiny);
  lcd.createChar(3, znakCelsius);
  lcd.createChar(4, znakDelta);

  digitalWrite(ledPinZelena, HIGH); // po inicializacii nastavi alarmovy vystup do LOG1, co znamena ze je vsetko OK, svieti zelena LED

  lcd.begin(16,2); // typ LCD
  lcd.clear(); // uvodne zmazanie LCD

// uvodny napis
lcd.setCursor(0,0);
lcd.print("ChladMonitor 2.1");
lcd.setCursor(0,1);
lcd.print("pablox.net 7/17");
delay(2000);
lcd.clear();
lcd.setCursor(0,0);
lcd.print(" UPDATE 9/2022  ");
lcd.setCursor(0,1);
lcd.print("MODBUS RTU INCL");
delay(2000);

holdingRegs[ID] = deviceID; // ulozi adresu zariadenia do registra

}


//=============================== HLAVNA SLUCKA ===============================

void loop(void)
{ 
MerajVypis();
PorovnajTeploty();
MODBUS(); // aktualizuje registre pre MODBUS RTU


// ovladanie LED diod v pripade aktivneho alarmu
if (alarm == 1) 
{ 
  digitalWrite(ledPinCervena, HIGH);
  digitalWrite(ledPinZelena, LOW);
}
else 
{
  if (chybaCidla == 0) digitalWrite(ledPinCervena, LOW); // ak je porucha na cidle, nedovoli zhasnut cervenu LED po skonceni alarmu
  digitalWrite(ledPinZelena, HIGH);
  digitalWrite(pipakPin, LOW);
}


// spusta sa hlavny alarm a caka sa bud na navrat teplot na normalne hodnoty alebo na stlacenie tlacidla
while (alarm == 1) //slucka ktora trva po dobu trvania alarmu
  {
  if (alarmPotvrdeny == 0) pipak(); else digitalWrite(pipakPin, LOW); // pipa bzuciak, ak je alam aktivny  
  digitalWrite(ledPinZelena, LOW); // zhasne zelenu LED
  tlacidloStav = digitalRead(tlacidloPin); //sem sa nacitava stav tlacidla

  // aktualizacia LCD, kontrola teplot a aktualizacia MODBUS registrov
  MerajVypis(); 
  PorovnajTeploty();
  MODBUS();

        
 // ak bolo stlacene tlacidlo a alarm je aktivny, odlozi sa alarm na nastaveny cas 
  if (tlacidloStav == 0 && alarm == 1) 
  {
    alarmPotvrdeny  = 1; // nastavi premennu o informacii, ze bol alarm odlozeny
    casOdlozeniaAlarmu = millis();
    Serial.println(" ALARM ODLOZENY ");
  }

  // ak bol alarm potvrdeny tlacidlom
if (alarmPotvrdeny == 1)
  {
    alarmNavratCas = (AlarmOdlozInterval-((millis() - casOdlozeniaAlarmu))) / 60000;  // prepocita zostavajuci cas odlozenia alarmu v miutach
    //vypise na LCD zostavajuci cas a znak hodin
    lcd.setCursor(13,1);
    lcd.write((uint8_t)2); //zobrazi znak hodin
    lcd.print(alarmNavratCas); 

  // ak uplynul cas odlozenia alarmu
      if (millis() - casOdlozeniaAlarmu >= AlarmOdlozInterval)
        {
          Serial.println("Skoncilo oneskorenie ALARMU");
          alarmPotvrdeny  = 0; // nuluje stavovu premennu
          lcd.setCursor(15,1); // zmaze hodinky
          lcd.print(" "); 
        }
  }   
  }  
}


//================================= POROVNANIE NAMERANYCH TEPLOT S PRIHLIADNUTIM NA NAVRATOVU HYSTEREZU  ================================
void PorovnajTeploty()
{
  // ak je prekrocena niektora teplota, nastavi premennu alarm na 1
  if ((teplotaChladnicka >= chladnickaAlarmTepl) or (teplotaMraznicka >= mraznickaAlarmTepl)) alarm = 1;

  // ak su obidve teploty v norme so zohladnenim hysterezy, nastavi premennu alarm na 0
  if ((teplotaChladnicka <= (chladnickaAlarmTepl-hystereza) && (teplotaMraznicka <= (mraznickaAlarmTepl - hystereza)))) alarm = 0;  
}



//================================= V TEJTO FUNKCII SA MERAJU TEPLOTY, VYPISUJU NA LCD, TESTUJU SA CIDLA /================================
void MerajVypis()
{
  sensors.requestTemperatures();
//CHLADNICKA
  teplotaChladnicka = sensors.getTempC(chladnicka_cidlo);
  lcd.clear();
  lcd.setCursor(0,0);
  lcd.print("Chlad: ");
  lcd.setCursor(6,0);
  lcd.write((uint8_t)0);
  lcd.print(teplotaChladnicka);
  lcd.write((uint8_t)3);
  lcd.write((uint8_t)4);
  lcd.print(teplotaChladnicka-chladnickaAlarmTepl); // pre informaciiu, o kolko je aktualna teplota rozdielna oproti prevadzkovej


//MRAZNICKA
  teplotaMraznicka = sensors.getTempC(mraznicka_cidlo);
  lcd.setCursor(0,1);
  lcd.print("Mraz: ");
  lcd.setCursor(6,1);
  lcd.write((uint8_t)0);
  lcd.print(teplotaMraznicka);
  lcd.write((uint8_t)3);
  lcd.write((uint8_t)4);
  lcd.print(teplotaMraznicka-mraznickaAlarmTepl); // pre informaciiu o kolko je aktualna teplota rozdielna oproti prevadzkovej

// testy cidiel ci je vsetko ok
if (teplotaChladnicka == -127.00) 
{
  chybaCidla = 1;
  lcd.clear();
  lcd.print("Cidlo CHLADNICKA");
  Serial.println("Cidlo CHLADNICKA");
//rozblika LED
  blikajLED();
} else chybaCidla = 0;

if (teplotaMraznicka == -127.00)
{
  chybaCidla = 1;
  lcd.clear();
  lcd.print("Cidlo MRAZNICKA");
  Serial.println("Cidlo MRAZNICKA");
  //rozblika LED
  blikajLED();
} else chybaCidla = 0;
//koniec testu cidiel

}

//================================= OBSLUHA SAMOVYBUZUJUCEHO PIEZO BZUCIAKA ================================
void pipak()
{
  unsigned long currentMillisPipak = millis();

  if (currentMillisPipak - previousMillisPipak >= pipakInterval) 
  { 
    previousMillisPipak = currentMillisPipak;
    
    if (pipakStav == LOW) { 
      pipakStav = HIGH;
    } else {
      pipakStav = LOW;
    }
    
    digitalWrite(pipakPin, pipakStav);
  }  
}

//================================= OBSLUHA BLIKANIA LED PRI PORUCHE CIDLA ================================
void blikajLED()
{
  unsigned long currentMillisLED = millis();

  if (currentMillisLED - previousMillisLED >= pipakInterval) 
  { 
    previousMillisLED = currentMillisLED;
    
    if (ledStav == LOW) { 
      ledStav = HIGH;
    } else {
      ledStav = LOW;
    }
    
    digitalWrite(ledPinCervena, ledStav);
  }  
}

//=================================== OBLUSHA A AKTUALIZACIA MODBUS REGISTROV ========================================

void MODBUS()
{
    holdingRegs[TOTAL_ERRORS] = modbus_update(holdingRegs);
    temp1 = teplotaChladnicka;
    temp2 = teplotaMraznicka;

    // ak je teplota zaporna, v registri SIGN nastavi 0, inak 1
    if (temp1 < 0) holdingRegs[TEMP1SIGN] = 0;  else holdingRegs[TEMP1SIGN] = 1;
    if (temp2 < 0) holdingRegs[TEMP2SIGN] = 0;  else holdingRegs[TEMP2SIGN] = 1;

    // ak je teplota zaporna, upravi hodnotu, aby nepretiekol register
    if (temp1 < 0) temp1 = (65535 - temp1) + 1;
    if (temp2 < 0) temp2 = (65535 - temp2) + 1;

    // ulozi teploty do registrov
    holdingRegs[TEMP1] = temp1;
    holdingRegs[TEMP2] = temp2;

    // nastavi registre OK, ALARM, SENSOR
    if (alarm == 1) holdingRegs[ALARM] = 1;  else holdingRegs[ALARM] = 0;
    if (digitalRead(ledPinZelena) == 1) holdingRegs[OK] = 1;  else holdingRegs[OK] = 0;
    if (chybaCidla == 1) holdingRegs[SENSOR] = 1;  else holdingRegs[SENSOR] = 0;   

}

//===========================================================================
