// MASTER PRE CITANIE Z MONITORA TEPLOTY V CHLADNICKE V2.0
// pouzity externy modul s MAX485. Pri pouziti samotneho MAX485 su potrebne PULLUP rezistory 10k medzi DI,DE/RE,RO a 5V
// nacitane data vypisuje na seriovej linke
// www.pablox.net 
// 9/2022

#include <ModbusRtu.h>
#include <SoftwareSerial.h>

#define address 7

// SW UART a riadenie MAX485
#define RX            3 //povodne 8       // pripojit na pin RO MAX485
#define TX            5 //povodne 9       // pripojit na pin DI MAX485
#define RS485_EN      4 //povodne 2       // pripojit na pin RE/DE MAX485

// pole pre nacitane data a pomocne premenne pre citanie dat
uint16_t au16data[16];
uint8_t u8state;
int CoilsNo = 0;
int node_id = 0;
int RegAdd = 0; 

int temp = 0;

// premenne pre citanie
int oneskorenie = 2000;
bool ok = 0;

SoftwareSerial SWserial(RX, TX); // SW UART pre MAX485

Modbus master(0, SWserial,RS485_EN); //priradi SW seriovu linku pre komunikaciu

modbus_t telegram;

unsigned long u32wait;

void setup() {
  Serial.begin(9600);  
  SWserial.begin(9600); //seriova linka pre RS485
  master.start(); 
  master.setTimeOut( 2000 ); // timeout
  u32wait = millis() + 200;
  u8state = 0;
}

//   400001 ID podstanice
//   400002 teplota v chladnicke, cely tvar bez znamienka a desatin
//   400003 znamienko teploty v chladnicke: 0: zaporna, 1: kladna
//   400004 teplota v mraznickee, cely tvar bez znamienka a desatin
//   400005 znamienko teploty v mraznicke: 0: zaporna, 1: kladna
//   400006 stav monitora (zelena LED): 1: OK, 0: ALARM
//   400007 stav alarmu po prekroceni teploty (cervena LED): 1: ALARM, 0: OK
//   400008 chyba cidla teploty: 0: cidlo OK, 1: chyba cidla

void loop() {
  
  node_id = address;
  
  // citanie z 8 registrov
  CoilsNo = 8;
  RegAdd = 0;    
  ok = 0;
  ziskajData();
  
  Serial.println("ID " + String(au16data[0]));

  // uprava znamienka teploty chladnicky na zaklade registra 400003
           temp = au16data[1];
           switch (au16data[2]){
            case 0:
              temp = temp*(-1);
              break;
            case 1:
              if (temp < 0) {temp*(-1);}
              break;
          }
  Serial.print("Teplota chladnicka ");
  Serial.println(temp);
  
    // uprava znamienka teploty mraznicky na zaklade registra 400004
           temp = au16data[3];
           switch (au16data[4]){
            case 0:
              temp = temp*(-1);
              break;
            case 1:
              if (temp < 0) {temp*(-1);}
              break;
          }
          
  Serial.print("Teplota mraznicka ");
  Serial.println(temp);
  Serial.println("OK " + String(au16data[5]));
  Serial.println("ALARM " + String(au16data[6]));
  Serial.println("SENSOR " + String(au16data[7]));
  delay(oneskorenie);
  Serial.println("---------------------------------------");
  
}

void ziskajData()
{
  while(ok == 0) {
  switch( u8state ) {
  case 0: 
    if (millis() > u32wait) u8state++; // wait state
    break;
  case 1: 
    
    telegram.u8id = node_id; // slave address 
    telegram.u8fct = 3; // function code (this one is registers read)
    telegram.u16RegAdd = RegAdd; // start address in slave
    telegram.u16CoilsNo = CoilsNo; // number of elements (coils or registers) to read
    telegram.au16reg = au16data; // pointer to a memory array in the Arduino

    master.query( telegram ); // send query (only once)
    u8state++;
    break;
  case 2:
    master.poll(); // check incoming messages
    if (master.getState() == COM_IDLE) {
      u8state = 0;
      u32wait = millis() + 2000;//2000;
      ok = 1;        
    break;
  }
}
  }
}
